@@ -25,6 +25,7 @@
 #include "value.h"
 #include "vector.h"
 #include "variable.h"
+#include "expressionwriter.h"
 #include "stringexpressionwriter.h"
 #include "htmlexpressionwriter.h"
 #include "mathmlexpressionwriter.h"
@@ -34,7 +35,8 @@
 #include "apply.h"
 #include "customobject.h"
 #include "localize.h"
-#include "matrix.h"
+
+static void print_dom(const QDomNode& in, int ind);
 
 using namespace Analitza;
 
@@ -93,7 +95,7 @@ Expression::Expression(const Expression & e)
 	: d(new ExpressionPrivate(0))
 {
 	d->m_err=e.d->m_err;
-	if(e.isCorrect() && e.d->m_tree)
+	if(e.isCorrect())
 		d->m_tree = e.d->m_tree->copy();
 }
 
@@ -129,15 +131,9 @@ Expression Expression::operator=(const Expression & e)
 
 bool Expression::setText(const QString & exp)
 {
-	if(exp.isEmpty()) {
-		delete d->m_tree;
-		d->m_tree = 0;
-		return true;
-	}
 	d->m_err.clear();
 	ExpLexer lex(exp);
 	ExpressionParser parser;
-	m_comments = parser.comments();
 	bool corr=parser.parse(&lex);
 	
 	if(corr)
@@ -300,16 +296,7 @@ static void variableDepth(Object* o, int& next, const QMap<QString, int>& scope,
 			Vector *v=(Vector*) o;
 			for(Vector::const_iterator it=v->constBegin(); it!=v->constEnd(); ++it)
 				variableDepth(*it, next, scope);
-		}	break;
-		case Object::matrix: {
-			Matrix *v=(Matrix*) o;
-			for(Matrix::const_iterator it=v->constBegin(); it!=v->constEnd(); ++it)
-				variableDepth(*it, next, scope);
-		}	break;
-		case Object::matrixrow: {
-			MatrixRow *v=(MatrixRow*) o;
-			for(MatrixRow::const_iterator it=v->constBegin(); it!=v->constEnd(); ++it)
-				variableDepth(*it, next, scope);
+			
 		}	break;
 		case Object::list: {
 			List *v=(List*) o;
@@ -460,12 +447,6 @@ Object* Expression::ExpressionPrivate::branch(const QDomElement& elem)
 		case Object::list:
 			ret=addElements<List>(new List, &elem);
 			break;
-		case Object::matrix:
-			ret=addElements<Matrix>(new Matrix, &elem);
-			break;
-		case Object::matrixrow:
-			ret=addElements<MatrixRow>(new MatrixRow, &elem);
-			break;
 		case Object::apply: {
 			Apply *a=addElements<Apply>(new Apply, &elem);
 			if(a && !check(a)) {
@@ -488,26 +469,26 @@ Object* Expression::ExpressionPrivate::branch(const QDomElement& elem)
 QString Expression::toHtml() const
 {
 	Q_ASSERT(isCorrect());
-	return HtmlExpressionWriter(d->m_tree).result().toString();
+	return HtmlExpressionWriter(d->m_tree).result();
 }
 
 QString Expression::toMathMLPresentation() const
 {
 	Q_ASSERT(isCorrect());
-	return MathMLPresentationExpressionWriter(d->m_tree).result().toString();
+	return MathMLPresentationExpressionWriter(d->m_tree).result();
 }
 
 QString Expression::toMathML() const
 {
 	Q_ASSERT(isCorrect());
-	return MathMLExpressionWriter(d->m_tree).result().toString();
+	return MathMLExpressionWriter(d->m_tree).result();
 }
 
 QString Expression::toString() const
 {
 	Q_ASSERT(isCorrect());
 	StringExpressionWriter s(d->m_tree);
-	return s.result().toString();
+	return s.result();
 }
 
 enum Object::ObjectType Expression::whatType(const QString& tag)
@@ -521,10 +502,6 @@ enum Object::ObjectType Expression::whatType(const QString& tag)
 		ret= Object::vector;
 	else if(tag=="list")
 		ret= Object::list;
-	else if(tag=="matrix")
-		ret= Object::matrix;
-	else if(tag=="matrixrow")
-		ret= Object::matrixrow;
 	else if(tag=="apply")
 		ret= Object::apply;
 	else if(Operator::toOperatorType(tag)!=0)
@@ -537,12 +514,7 @@ enum Object::ObjectType Expression::whatType(const QString& tag)
 
 bool Expression::operator==(const Expression & e) const
 {
-	return AnalitzaUtils::equalTree(e.d->m_tree, d->m_tree);
-}
-
-bool Expression::operator!=(const Expression& e) const
-{
-	return !(e==*this);
+	return e.d->m_tree && d->m_tree && AnalitzaUtils::equalTree(e.d->m_tree, d->m_tree);
 }
 
 void Expression::clear()
@@ -554,7 +526,7 @@ void Expression::clear()
 
 bool Expression::isCorrect() const
 {
-	return !(d || d->m_tree) || d->m_err.isEmpty();
+	return d && d->m_tree && d->m_err.isEmpty();
 }
 
 QStringList Expression::bvarList() const
@@ -605,95 +577,30 @@ bool Expression::isString() const
 			&& static_cast<const Cn*>(static_cast<const List*>(d->m_tree)->at(0))->isCharacter();
 }
 
-//if it's a math element, then return the (only) son
-Object* actualRoot(Object* o)
-{
-	if(o && o->isContainer()) {
-		Container* c = (Container*) o;
-		if(c->containerType()==Container::math) {
-			return c->m_params[0];
-		}
-	}
-	return o;
-}
-
 bool Expression::isLambda() const
 {
-	Object* o = actualRoot(d->m_tree);
-	if(o && o->isContainer()) {
-		Container* c = (Container*) o;
+	if(d->m_tree && d->m_tree->type()==Object::container) {
+		Container* c = (Container*) d->m_tree;
+		if(c->containerType()==Container::math) {
+			Container *c1 = (Container*) c->m_params.first();
+			return c1->type()==Object::container && c1->containerType()==Container::lambda;
+		}
 		return c->containerType()==Container::lambda;
 	}
 	return false;
 }
 
-bool Expression::isDeclaration() const
-{
-	Object* o = actualRoot(d->m_tree);
-	if(o && o->isContainer()) {
-		Container* c = (Container*) o;
-		return c->containerType() == Container::declare;
-	}
-	return false;
-}
-
-QString Expression::name() const
-{
-	Object* o = actualRoot(d->m_tree);
-	if(o && o->isContainer()) {
-		Container* c = (Container*) o;
-		if(c->containerType() == Container::declare)
-			return static_cast<const Ci*>(c->m_params[0])->name();
-	}
-	return QString();
-}
-
-Expression Expression::declarationValue() const
-{
-	Object* o = actualRoot(d->m_tree);
-	if(o && o->isContainer()) {
-		Container* c = (Container*) o;
-		if(c->containerType() == Container::declare)
-			return Expression(c->m_params[1]->copy());
-	}
-	return Expression();
-}
-
-bool Expression::isEquation() const
-{
-	Object* o = actualRoot(d->m_tree);
-	if(o && o->isApply()) {
-		Apply *c1 = (Apply*) o;
-		return c1->firstOperator().operatorType()==Operator::eq;
-	}
-	return false;
-}
-
-Expression Expression::equationToFunction() const
-{
-	Object* o = actualRoot(d->m_tree);
-	if(o && o->isApply()) {
-		Apply *c1 = (Apply*) o;
-		if(c1->firstOperator().operatorType()==Operator::eq) {
-			Container* c = new Container(Container::math);
-			Apply* a = new Apply;
-			a->appendBranch(new Operator(Operator::minus));
-			a->appendBranch(c1->at(0)->copy());
-			a->appendBranch(c1->at(1)->copy());
-			c->appendBranch(a);
-			return Expression(c);
-		}
-	}
-	return *this;
-}
-
 QList<Ci*> Expression::parameters() const
 {
 	QList<Ci*> ret;
-	Object* o = actualRoot(d->m_tree);
-	if(o && o->isContainer()) {
-		Container* c1 = (Container*) o;
-		return c1->bvarCi();
+	if(d->m_tree && d->m_tree->type()==Object::container) {
+		Container* c = (Container*) d->m_tree;
+		if(c->containerType()==Container::math) {
+			Container *c1 = (Container*) c->m_params.first();
+			if(c1->type()==Object::container && c1->containerType()==Container::lambda)
+				return c1->bvarCi();
+		}
+		return c->bvarCi();
 	}
 	return ret;
 }
@@ -702,10 +609,12 @@ Expression Expression::lambdaBody() const
 {
 	Q_ASSERT(isLambda());
 	Object *ret=0;
-	Object* ob = actualRoot(d->m_tree);
-	if(ob->isContainer()) {
-		Container* c = (Container*) ob;
-		Q_ASSERT(c->containerType()==Container::lambda);
+	if(d->m_tree) {
+		Container* c = (Container*) d->m_tree;
+		if(c->containerType()==Container::math) {
+			Container *c1 = (Container*) c->m_params.first();
+			ret = c1->m_params.last();
+		} else
 		ret = c->m_params.last();
 	}
 	
@@ -716,14 +625,26 @@ Expression Expression::lambdaBody() const
 
 bool Expression::isVector() const
 {
-	Object* o = actualRoot(d->m_tree);
-	return o && o->type()==Container::vector;
+	if(d->m_tree) {
+		if(d->m_tree->isContainer()) {
+			Container *c = (Container*) d->m_tree;
+			return c->containerType()==Container::math && c->m_params.first()->type()==Object::vector;
+		} else
+			return d->m_tree->type()==Container::vector;
+	}
+	return false;
 }
 
 bool Expression::isList() const
 {
-	Object* o = actualRoot(d->m_tree);
-	return o && o->type()==Container::list;
+	if(d->m_tree) {
+		if(d->m_tree->isContainer()) {
+			Container *c = (Container*) d->m_tree;
+			return c->containerType()==Container::math && c->m_params.first()->type()==Object::list;
+		} else
+			return d->m_tree->type()==Container::list;
+	}
+	return false;
 }
 
 QList<Expression> Expression::toExpressionList() const
@@ -733,8 +654,13 @@ QList<Expression> Expression::toExpressionList() const
 		return QList<Expression>();
 	
 	QList<Expression> ret;
-	const Object* o=actualRoot(d->m_tree);
-
+	const Object* o=d->m_tree;
+	if(d->m_tree->isContainer()) {
+		const Container *c = (const Container*) d->m_tree;
+		Q_ASSERT(c->containerType()==Container::math);
+		o=c->m_params.first();
+	}
+	
 	if(isvector) {
 		const Vector* v = (const Vector*) o;
 		for(Vector::const_iterator it=v->constBegin(), itEnd=v->constEnd(); it!=itEnd; ++it) {
@@ -757,24 +683,18 @@ QList<Expression> Expression::toExpressionList() const
 Expression Expression::elementAt(int position) const
 {
 	Q_ASSERT(isVector());
-	Object* o = actualRoot(d->m_tree);
-	Q_ASSERT(o);
-	Vector *v=static_cast<Vector*>(o);
+	Vector *v=0;
+	if(d->m_tree) {
+		if(d->m_tree->isContainer()) {
+			Container *c = (Container*) d->m_tree;
+			v=static_cast<Vector*>(c->m_params.first());
+		} else
+			v=static_cast<Vector*>(d->m_tree);
+	}
 	
 	return Expression(v->at(position)->copy());
 }
-
-void Expression::setElementAt(int position, const Expression& exp)
-{
-	Q_ASSERT(isVector());
-	Object* o = actualRoot(d->m_tree);
-	Q_ASSERT(o);
-	Vector *v=static_cast<Vector*>(o);
-	
-	delete v->at(position);
-	v->setAt(position, exp.tree()->copy());
-}
-
+// 
 QStringList Expression::error() const
 {
 	return d->m_err;
@@ -846,8 +766,6 @@ static void renameTree(Object* o, int depth, const QString& newName)
 		case Object::container: { CHECK_TYPE(Container); } break;
 		case Object::list: { CHECK_TYPE(List); } break;
 		case Object::vector: { CHECK_TYPE(Vector); } break;
-		case Object::matrix: { CHECK_TYPE(Matrix); } break;
-		case Object::matrixrow: { CHECK_TYPE(MatrixRow); } break;
 		
 		case Object::none:
 		case Object::value:
@@ -908,8 +826,3 @@ bool Expression::isCompleteExpression(const QString& exp, bool justempty)
 	ExpLexer lex(exp);
 	return lex.isCompleteExpression(justempty);
 }
-
-QStringList Expression::comments() const
-{
-	return m_comments;
-}
