@@ -26,7 +26,6 @@
 #include "variable.h"
 #include "apply.h"
 #include <analitza/analitzautils.h>
-#include "matrix.h"
 
 using namespace Analitza;
 
@@ -38,7 +37,7 @@ QStringList convertElements(T it, const T& itEnd, MathMLPresentationExpressionWr
 {
 	QStringList elems;
 	for(; it!=itEnd; ++it) {
-		elems += (*it)->accept(w).toString();
+		elems += (*it)->visit(w);
 	}
 	return elems;
 }
@@ -98,7 +97,7 @@ QString root(const Apply* c, MathMLPresentationExpressionWriter* w)
 {
 	Cn two(2);
 	if(AnalitzaUtils::equalTree(c->values()[1], &two))
-		return "<msqrt>"+(*c->firstValue())->accept(w).toString()+"</msqrt>";
+		return "<msqrt>"+(*c->firstValue())->visit(w)+"</msqrt>";
 	else
 		return "<mroot>"+convertElements<Apply::const_iterator>(c->firstValue(), c->constEnd(), w).join(QString())+"</mroot>";
 }
@@ -140,9 +139,9 @@ QString function(const Apply* c, MathMLPresentationExpressionWriter* w)
 {
 	QString ret="<mrow>";
 	foreach(const Ci* bvar, c->bvarCi())
-		ret+=bvar->accept(w).toString();
+		ret+=bvar->visit(w);
 	foreach(const Object* o, c->values())
-		ret+=o->accept(w).toString();
+		ret+=o->visit(w);
 	ret+="</mrow>";
 	return ret;
 }
@@ -188,20 +187,20 @@ MathMLPresentationExpressionWriter::operatorToString
 
 MathMLPresentationExpressionWriter::MathMLPresentationExpressionWriter(const Object* o)
 {
-	m_result=o->accept(this);
+	m_result=o->visit(this);
 }
 
-QVariant MathMLPresentationExpressionWriter::visit(const Ci* var)
+QString MathMLPresentationExpressionWriter::accept(const Ci* var)
 {
-	return QString("<mi>"+var->name()+"</mi>");
+	return "<mi>"+var->name()+"</mi>";
 }
 
-QVariant MathMLPresentationExpressionWriter::visit(const Operator* op)
+QString MathMLPresentationExpressionWriter::accept(const Operator* op)
 {
 	return op->name();
 }
 
-QVariant MathMLPresentationExpressionWriter::visit(const Cn* val)
+QString MathMLPresentationExpressionWriter::accept(const Cn* val)
 {
 	if(val->isBoolean()) {
 		if(val->isTrue())
@@ -224,17 +223,17 @@ QString piecewise(const Container* c, MathMLPresentationExpressionWriter* w)
 		if(piece->containerType()==Container::piece) {
 			ret += "<mtr>"
 			"<mtd>"
-				+piece->m_params.first()->accept(w).toString()+
+				+piece->m_params.first()->visit(w)+
 			"</mtd>"
 			"<mtd>"
 				"<mtext>if </mtext>"
-				+piece->m_params.last()->accept(w).toString()+
+				+piece->m_params.last()->visit(w)+
 			"</mtd>"
 			"</mtr>";
 		} else {
 			ret += "<mtr>"
 			"<mtd>"
-				+piece->m_params.first()->accept(w).toString()+
+				+piece->m_params.first()->visit(w)+
 			"</mtd>"
 			"<mtd>"
 				"<mtext>otherwise</mtext>"
@@ -251,14 +250,14 @@ QString lambda(const Container* c, MathMLPresentationExpressionWriter* w)
 {
 	QString ret="<mrow>";
 	foreach(const Ci* bvar, c->bvarCi())
-		ret+=bvar->accept(w).toString();
+		ret+=bvar->visit(w);
 	ret+="<mo>&RightArrow;</mo>";
-	ret+=c->m_params.last()->accept(w).toString();
+	ret+=c->m_params.last()->visit(w);
 	ret+="</mrow>";
 	return ret;
 }
 
-QVariant MathMLPresentationExpressionWriter::visit(const Container* c)
+QString MathMLPresentationExpressionWriter::accept(const Container* c)
 {
 	QString ret;
 // 	objectWalker(c);
@@ -291,27 +290,17 @@ QVariant MathMLPresentationExpressionWriter::visit(const Container* c)
 	return ret;
 }
 
-QVariant MathMLPresentationExpressionWriter::visit(const Vector* var)
+QString MathMLPresentationExpressionWriter::accept(const Vector* var)
 {
-	return QString("<mrow><mo>&lt;</mo>"+convertElements(var->constBegin(), var->constEnd(), this).join("<mo>,</mo>")+"<mo>&gt;</mo></mrow>");
+	return "<mrow><mo>&lt;</mo>"+convertElements(var->constBegin(), var->constEnd(), this).join("<mo>,</mo>")+"<mo>&gt;</mo></mrow>";
 }
 
-QVariant MathMLPresentationExpressionWriter::visit(const List* var)
+QString MathMLPresentationExpressionWriter::accept(const List* var)
 {
-	return QString("<mrow><mo>[</mo>"+convertElements(var->constBegin(), var->constEnd(), this).join("<mo>,</mo>")+"<mo>]</mo></mrow>");
+	return "<mrow><mo>[</mo>"+convertElements(var->constBegin(), var->constEnd(), this).join("<mo>,</mo>")+"<mo>]</mo></mrow>";
 }
 
-QVariant MathMLPresentationExpressionWriter::visit(const Matrix* m)
-{
-	return QString("<mrow><mo>[</mo>"+convertElements(m->constBegin(), m->constEnd(), this).join("<mo>,</mo>")+"<mo>]</mo></mrow>");
-}
-
-QVariant MathMLPresentationExpressionWriter::visit(const MatrixRow* m)
-{
-	return QString("<mrow><mo>[</mo>"+convertElements(m->constBegin(), m->constEnd(), this).join("<mo>,</mo>")+"<mo>]</mo></mrow>");
-}
-
-QVariant Analitza::MathMLPresentationExpressionWriter::visit(const Analitza::Apply* a)
+QString Analitza::MathMLPresentationExpressionWriter::accept ( const Analitza::Apply* a )
 {
 	QString ret;
 	Operator op=a->firstOperator();
@@ -329,11 +318,11 @@ QVariant Analitza::MathMLPresentationExpressionWriter::visit(const Analitza::App
 			const Object *ul=a->ulimit(), *dl=a->dlimit();
 			if(ul || dl) {
 				bvars += "<mo>=</mo>";
-				if(dl) bvars += dl->accept(this).toString();
+				if(dl) bvars += dl->visit(this);
 				bvars += "<mo>..</mo>";
-				if(ul) bvars += ul->accept(this).toString();
+				if(ul) bvars += ul->visit(this);
 			} else if(a->domain())
-				bvars += "<mo>@</mo>" + a->domain()->accept(this).toString();
+				bvars += "<mo>@</mo>" + a->domain()->visit(this);
 			bvars="<mo>:</mo>"+bvars;
 		}
 		
@@ -347,7 +336,7 @@ QVariant Analitza::MathMLPresentationExpressionWriter::visit(const Analitza::App
 	return ret;
 }
 
-QVariant MathMLPresentationExpressionWriter::visit(const CustomObject*)
+QString MathMLPresentationExpressionWriter::accept(const CustomObject*)
 {
-	return QString("<!-- custom object -->");
+	return "<!-- custom object -->";
 }
